#include <AFMotor.h>
#include <QTRSensors.h> //подключение библиотеки

#define NUM_SENSORS             8  // количество датчиков
#define NUM_SAMPLES_PER_SENSOR  4  // количество датчиков с одной стороны


#define KP 0.08//коэфициент пропорциональный
#define KD 0.3//коэфициент дифференциальный
#define KI 0.0005//коэфициент интегральный

#define HIST_COUNT 500//сколько запоминать значений
#define M1_DEFAULT_SPEED 255//стандарт скорость 1 мотора
#define M2_DEFAULT_SPEED 255//стандарт скорость 2 мотора
#define M1_MAX_SPEED 255//максимальная скорость 
#define M2_MAX_SPEED 255//максимальная скорость 
#define M1_MIN_SPEED -30//минимальная скорость 
#define M2_MIN_SPEED -30//минимальная скорость 

int hist[HIST_COUNT];//запоминать значение в этот архив
unsigned int hist_pos = 0;//беззнаковое значение 

/*struct MOTOR  //задаем моторы через структкру
{
  int in1;      // INPUT1
  int in2;      // INPUT2
  int enable;   // ENABLE
};

MOTOR MOTOR1 = { 7, 8, 10 };// вперед назад енейбл первого мотора
MOTOR MOTOR2 = { 3, 2, 9 };// вперед назад енейбл второго мотора
*/
AF_DCMotor motorr(1);
AF_DCMotor motorl(2);
QTRSensorsAnalog qtra((unsigned char[]) {7, 6, 5, 4, 3, 2, 1, 0}, 
  NUM_SENSORS, NUM_SAMPLES_PER_SENSOR);

//считываем значение с сенсоров
/*QTRSensorsAnalog qtra((unsigned char[]) {0, 3, 7}, 
  3, NUM_SAMPLES_PER_SENSOR);*/
  
unsigned int sensorValues[NUM_SENSORS];

void setup()
{
  Serial.begin(9600);
  pinMode(12, INPUT_PULLUP);
  delay(500);
  pinMode(13, OUTPUT);
  digitalWrite(13, HIGH);
  for (int i = 0; i < HIST_COUNT; i++)// обнуляем историю
  {
    hist[i] = 0;
  }
  for (int i = 0; i < 200; i++) 
  {
    qtra.calibrate(); //калибруем датчики
  }
  digitalWrite(13, LOW);  

  for (int i = 0; i < NUM_SENSORS; i++)
  {
    Serial.print(qtra.calibratedMinimumOn[i]);// ищем минимум
    Serial.print(' ');
  }
  Serial.println();
  
  for (int i = 0; i < NUM_SENSORS; i++)
  {
    Serial.print(qtra.calibratedMaximumOn[i]);// ищем максимум
    Serial.print(' ');
  }
  
  Serial.println();
  Serial.println();
  Serial.println("Wait for button...");
  
  while(digitalRead(12) != 0)//ждем кнопку
    delay(50);
}


int lastError = 0; //текущая ошибка
int  last_proportional = 0;// текущ. пропорц
int integral = 0;// текущ интеграл
void set_motors(int motor1speed, int motor2speed);

void loop()
{
  
  unsigned int sensors[NUM_SENSORS];
  int position = qtra.readLine(sensors);
  int error = position - 500 * (NUM_SENSORS-1);

  if(hist_pos == HIST_COUNT)
    hist_pos = 0;
  hist[hist_pos] = error;
  hist_pos++;

  double hist_err = 0;
  for (int i = 0; i < HIST_COUNT; i++)
  {
    hist_err += hist[i];
  }
  
  double correction = (double)KP * (double)error + (double)KD * (double)(error - lastError) + (double)KI * (double)hist_err;
  lastError = error;

  if (motorr.setSpeed >= 0) {
   motorr.run(FORWARD);
}
else {
  motorr.run(BACKWARD);
  rightMotorSpeed=rightMotorSpeed-1;
}
if (motorl.setSpeed >= 0) {
   motorl.run(FORWARD);
}
else {
  motorl.run(BACKWARD);
  leftMotorSpeed=leftMotorSpeed-1;
}
  int leftMotorSpeed = M1_DEFAULT_SPEED + (int)correction;
  int rightMotorSpeed = M2_DEFAULT_SPEED - (int)correction;

  Serial.println(correction);
  set_motors(leftMotorSpeed, rightMotorSpeed);
}
void set_motors(int motor1speed, int motor2speed)
{
  if (motor1speed > M1_MAX_SPEED) motor1speed = M1_MAX_SPEED; 
  if (motor2speed > M2_MAX_SPEED) motor2speed = M2_MAX_SPEED; 
  if (motor1speed < M1_MIN_SPEED) motor1speed = M1_MIN_SPEED;
  if (motor2speed < M2_MIN_SPEED) motor2speed = M2_MIN_SPEED; 
  motorr.setSpeed=leftMotorSpeed;
  motorl.setSpeed=rightMotorSpeed;
}





